/*
 * Analyseur Lexical
 * Auteur : P. Chour - 2018-02-09 - V1
 */
 #include "pc_lex.h"


/**********************************
 * ROUTINES ANALYSEUR LEXICAL
**********************************/
/*
 * Positionne le pointeur sur le premier caractère différent d'un séparateur
 */
void LexFiltre(LexBuf* B)
{
  B->Lfin = B->Lfin || (B->Lindex >=  B->Lbuf.length());
  while ((!(B->Lfin)) && (B->Lindex < B->Lbuf.length())) {
    if ((B->Lbuf[B->Lindex]==' ')  || (B->Lbuf[B->Lindex]==',')) {
      B->Lindex = B->Lindex+1;
      B->Lfin = (B->Lindex >= B->Lbuf.length());
    } else {
      B->Lfin = (B->Lbuf[B->Lindex]==RC);
      break;
    }
  }
}

/*
 * Saute un caractère et filtre les séparateurs
 */
void LexSkip(LexBuf* B, char C)
{
  LexFiltre(B);
  if (!(B->Lfin)) {
    if (B->Lbuf[B->Lindex] == C) {
      B->Lindex = B->Lindex+1;
      LexFiltre(B);
    }
  }
}

/* 
* réinitialise l'analyseur lexical
*/
void LexClear(LexBuf* B)
{
  B->Lbuf = "";
  B->Lindex = 0;
  B->Lfin = true;
}

/* 
* réinitialise l'analyseur lexical
*/
void LexStart(LexBuf* B)
{
  B->Lindex = 0;
  B->Lfin = false;
  LexFiltre(B);
}

/* 
* Rend un mot (alphabétique converti en minuscule)
*/
String LexIdent(LexBuf* B)
{
  String S = "";
  LexFiltre(B);
  while ((!(B->Lfin)) && (isAlpha(B->Lbuf[B->Lindex]))) {
    S.concat(B->Lbuf[B->Lindex]);
    B->Lindex = B->Lindex+1;
    B->Lfin = (B->Lindex >= B->Lbuf.length());
  }
  S.toUpperCase();
  return S;
}

/* 
* Rend un entier sous forme de string et sa valeur V en binaire (ou inchangé si string vide)
*/
String LexInt(LexBuf* B, long* V)
{
  String S = "";
  LexFiltre(B);
  while ((!(B->Lfin)) && (isDigit(B->Lbuf[B->Lindex]))) {
    S.concat(B->Lbuf[B->Lindex]);
    B->Lindex = B->Lindex+1;
    B->Lfin = (B->Lindex >= B->Lbuf.length());
  }
  if (S != "") {*V = S.toInt();}
  return S;
}

/* 
* Rend un hexa sous forme de string et sa valeur V en binaire (ou inchangé si string vide)
*/
String LexHexa(LexBuf* B, long* V)
{
  String S = "";
  int i;
  int v;
  LexFiltre(B);
  while ((!(B->Lfin)) && (isHexadecimalDigit(B->Lbuf[B->Lindex]))) {
    S.concat(B->Lbuf[B->Lindex]);
    B->Lindex = B->Lindex+1;
    B->Lfin = (B->Lindex >= B->Lbuf.length());
  }
  S.toUpperCase();
  if (S != "") {
	  *V = 0;
	  for (i=0; i< S.length(); i++) { // pour mémoire, on est sur d'avoir des caractères '0'..'9' ou 'A'..'F'
		v = int(S.charAt(i));
		if ((v >= 48) && (v <= 57)) {
			*V = (*V << 4) | (v & 0x0F);
		} else {
			*V = (*V << 4) | (v - 55);
		}
	  }	
  }
  return S;
}


/*
 * Rend "-" si signe - présent et filtre les séparateurs suivants. Rend vide sinon
 */
String LexSigne(LexBuf* B)
{
  String Signe = "";
  LexFiltre(B);
  if (!(B->Lfin)) {
     if (B->Lbuf[B->Lindex] == '-') {
      Signe = "-";
      B->Lindex = B->Lindex+1;
      LexFiltre(B);
     }
  }
  return Signe;
}
/* 
* Rend un Reel sous forme de string. V = valeur réelle si String non vide. Inchangé sinon
*/
String LexReal(LexBuf* B, float* V)
{
  String S;
  bool Point;
  Point = false;
  S = LexSigne(B);
  while ((!(B->Lfin)) && ((isDigit(B->Lbuf[B->Lindex])) || (B->Lbuf[B->Lindex]=='.')) ) {
    S.concat(B->Lbuf[B->Lindex]);
    B->Lindex = B->Lindex+1;
    B->Lfin = (B->Lindex >= B->Lbuf.length());
    if (B->Lbuf[B->Lindex]=='.') {
      if (Point) {
        S = "";
        break;
      } else {
        Point = true;
      }
    }
  }
  if (S != "") {*V = S.toFloat();}
  return S;
}

/*
 * Construit une string pour pointer une erreur (Lindex = endroit où il y a l'erreur) et l'affiche
 */
 void LexError(LexBuf* B)
 {
  String S = "";
  int i;
  for (i=0; i < B->Lindex-1; i++) {
    S.concat(' ');
  }
  Serial.println(B->Lbuf);  
  S.concat("+----- erreur de syntaxe");
  Serial.println(S);
}


