/*
 * Analyseur Lexical
 * Auteur : P. Chour - 2018-02-09 - V1
 */

#ifndef pc_lex.h
#define pc_lex.h
#include <arduino.h>

typedef struct LexBuf LexBuf; // pour analyseur lexical

const byte RC = 0x0D;               // Retour chariot


/*
 * déclarations pour analyse lexicale
 */
const int Lbufmax = 50;   // Taille max d'un buffer pour l'analyseur lexical
struct LexBuf             // Structure pour l'analyseur lexical
{
  String  Lbuf = String(Lbufmax);  // Le buffer qui contient la commande à analyser
  int   Lindex;                    // Un index qui indique où on en est de l'analyse lexicale
  bool  Lfin;                      // Un indicateur qui indique qu'il n'y a plus rien à analyser dans le buffer
};


/**********************************
 * ROUTINES ANALYSEUR LEXICAL
**********************************/
/*
 * Positionne le pointeur sur le premier caractère différent d'un séparateur
 */
void LexFiltre(LexBuf* B);

/*
 * Saute un caractère et filtre les séparateurs
 */
void LexSkip(LexBuf* B, char C);

/* 
* réinitialise l'analyseur lexical
*/
void LexClear(LexBuf* B);

/* 
* réinitialise l'analyseur lexical
*/
void LexStart(LexBuf* B);

/* 
* Rend un mot (alphabétique converti en minuscule)
*/
String LexIdent(LexBuf* B);

/* 
* Rend un entier sous forme de string et sa valeur V en binaire (ou inchangé si string vide)
*/
String LexInt(LexBuf* B, long* V);

/* 
* Rend un hexa sous forme de string et sa valeur V en binaire (ou inchangé si string vide)
*/
String LexHexa(LexBuf* B, long* V);

/*
 * Rend "-" si signe - présent et filtre les séparateurs suivants. Rend vide sinon
 */
String LexSigne(LexBuf* B);

/* 
* Rend un Reel sous forme de string. V = valeur réelle si String non vide. Inchangé sinon
*/
String LexReal(LexBuf* B, float* V);

/*
 * Construit une string pour pointer une erreur (Lindex = endroit où il y a l'erreur) et l'affiche
 */
 void LexError(LexBuf* B);

#endif

